/*      -*- OpenSAF  -*-
 *
 * (C) Copyright 2008 The OpenSAF Foundation
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. This file and program are licensed
 * under the GNU Lesser General Public License Version 2.1, February 1999.
 * The complete license can be accessed from the following location:
 * http://opensource.org/licenses/lgpl-license.php
 * See the Copying file included with the OpenSAF distribution for full
 * licensing terms.
 *
 * Author(s): Ericsson AB
 *
 */
#include "osaf/apitest/utest.h"
#include "osaf/apitest/util.h"
#include "ntf/apitest/tet_ntf.h"
#include "ntf/apitest/tet_ntf_common.h"
#include "ntf/apitest/ntf_api_with_try_again.h"

void saNtfNotificationSubscribe_01(void) {
  SaNtfHandleT ntfHandle;
  SaNtfAlarmNotificationFilterT myAlarmFilter;
  SaNtfNotificationTypeFilterHandlesT myNotificationFilterHandles;

  saNotificationAllocationParamsT myNotificationAllocationParams;
  saNotificationFilterAllocationParamsT
      myNotificationFilterAllocationParams;
  saNotificationParamsT myNotificationParams;

  fillInDefaultValues(&myNotificationAllocationParams,
          &myNotificationFilterAllocationParams,
          &myNotificationParams);

  safassert(NtfTest::saNtfInitialize(&ntfHandle, &ntfCallbacks, &ntfVersion),
      SA_AIS_OK);

  safassert(
      saNtfAlarmNotificationFilterAllocate(
    ntfHandle, &myAlarmFilter,
    myNotificationFilterAllocationParams.numEventTypes,
    myNotificationFilterAllocationParams.numNotificationObjects,
    myNotificationFilterAllocationParams.numNotifyingObjects,
    myNotificationFilterAllocationParams.numNotificationClassIds,
    myNotificationFilterAllocationParams.numProbableCauses,
    myNotificationFilterAllocationParams.numPerceivedSeverities,
    myNotificationFilterAllocationParams.numTrends),
      SA_AIS_OK);
  /* Set perceived severities */
  myAlarmFilter.perceivedSeverities[0] = SA_NTF_SEVERITY_WARNING;
  myAlarmFilter.perceivedSeverities[1] = SA_NTF_SEVERITY_CLEARED;

  /* Initialize filter handles */
  myNotificationFilterHandles.alarmFilterHandle =
      myAlarmFilter.notificationFilterHandle;
  myNotificationFilterHandles.attributeChangeFilterHandle = 0;
  myNotificationFilterHandles.objectCreateDeleteFilterHandle = 0;
  myNotificationFilterHandles.securityAlarmFilterHandle = 0;
  myNotificationFilterHandles.stateChangeFilterHandle = 0;

  rc = NtfTest::saNtfNotificationSubscribe(&myNotificationFilterHandles, 4);
  safassert(NtfTest::saNtfNotificationUnsubscribe(4), SA_AIS_OK);
  safassert(saNtfNotificationFilterFree(
          myNotificationFilterHandles.alarmFilterHandle),
      SA_AIS_OK);

  safassert(NtfTest::saNtfFinalize(ntfHandle), SA_AIS_OK);
  free(myNotificationParams
     .additionalText); /* allocated in fillInDefaultValues */
  test_validate(rc, SA_AIS_OK);
}

/* Test NULL handle parameter */
void saNtfNotificationSubscribe_02(void) {
  rc = NtfTest::saNtfNotificationSubscribe(NULL, 4);
  test_validate(rc, SA_AIS_ERR_INVALID_PARAM);
}

/* Test all filter handles set to NULL */
void saNtfNotificationSubscribe_03(void) {
  SaNtfHandleT ntfHandle;
  SaNtfNotificationTypeFilterHandlesT myNotificationFilterHandles;

  saNotificationAllocationParamsT myNotificationAllocationParams;
  saNotificationFilterAllocationParamsT
      myNotificationFilterAllocationParams;
  saNotificationParamsT myNotificationParams;

  fillInDefaultValues(&myNotificationAllocationParams,
          &myNotificationFilterAllocationParams,
          &myNotificationParams);

  safassert(NtfTest::saNtfInitialize(&ntfHandle, &ntfCallbacks, &ntfVersion),
      SA_AIS_OK);

  /* Initialize filter handles */
  myNotificationFilterHandles.alarmFilterHandle =
      SA_NTF_FILTER_HANDLE_NULL;
  myNotificationFilterHandles.attributeChangeFilterHandle =
      SA_NTF_FILTER_HANDLE_NULL;
  myNotificationFilterHandles.objectCreateDeleteFilterHandle =
      SA_NTF_FILTER_HANDLE_NULL;
  myNotificationFilterHandles.securityAlarmFilterHandle =
      SA_NTF_FILTER_HANDLE_NULL;
  myNotificationFilterHandles.stateChangeFilterHandle =
      SA_NTF_FILTER_HANDLE_NULL;

  rc = NtfTest::saNtfNotificationSubscribe(&myNotificationFilterHandles, 4);
  safassert(NtfTest::saNtfNotificationUnsubscribe(4), SA_AIS_ERR_NOT_EXIST);

  safassert(NtfTest::saNtfFinalize(ntfHandle), SA_AIS_OK);
  free(myNotificationParams
     .additionalText); /* allocated in fillInDefaultValues */
  test_validate(rc, SA_AIS_ERR_INVALID_PARAM);
}

/* Test already existing subscriptionId */
void saNtfNotificationSubscribe_04(void) {
  SaNtfHandleT ntfHandle;
  SaNtfAlarmNotificationFilterT myAlarmFilter;
  SaNtfNotificationTypeFilterHandlesT myNotificationFilterHandles;
  SaNtfSubscriptionIdT subscId = 0;
  saNotificationAllocationParamsT myNotificationAllocationParams;
  saNotificationFilterAllocationParamsT
      myNotificationFilterAllocationParams;
  saNotificationParamsT myNotificationParams;

  fillInDefaultValues(&myNotificationAllocationParams,
          &myNotificationFilterAllocationParams,
          &myNotificationParams);

  safassert(NtfTest::saNtfInitialize(&ntfHandle, &ntfCallbacks, &ntfVersion),
      SA_AIS_OK);

  safassert(
      saNtfAlarmNotificationFilterAllocate(
    ntfHandle, &myAlarmFilter,
    myNotificationFilterAllocationParams.numEventTypes,
    myNotificationFilterAllocationParams.numNotificationObjects,
    myNotificationFilterAllocationParams.numNotifyingObjects,
    myNotificationFilterAllocationParams.numNotificationClassIds,
    myNotificationFilterAllocationParams.numProbableCauses,
    myNotificationFilterAllocationParams.numPerceivedSeverities,
    myNotificationFilterAllocationParams.numTrends),
      SA_AIS_OK);
  /* Set perceived severities */
  myAlarmFilter.perceivedSeverities[0] = SA_NTF_SEVERITY_WARNING;
  myAlarmFilter.perceivedSeverities[1] = SA_NTF_SEVERITY_CLEARED;

  /* Initialize filter handles */
  myNotificationFilterHandles.alarmFilterHandle =
      myAlarmFilter.notificationFilterHandle;
  myNotificationFilterHandles.attributeChangeFilterHandle = 0;
  myNotificationFilterHandles.objectCreateDeleteFilterHandle = 0;
  myNotificationFilterHandles.securityAlarmFilterHandle = 0;
  myNotificationFilterHandles.stateChangeFilterHandle = 0;

  safassert(NtfTest::saNtfNotificationSubscribe(
      &myNotificationFilterHandles, subscId), SA_AIS_OK);
  rc = NtfTest::saNtfNotificationSubscribe(
      &myNotificationFilterHandles, subscId);
  /* No need to saNtfNotificationUnsubscribe since the subscribe failed */

  safassert(NtfTest::saNtfNotificationUnsubscribe(subscId), SA_AIS_OK);
  safassert(saNtfNotificationFilterFree(
          myNotificationFilterHandles.alarmFilterHandle),
      SA_AIS_OK);

  safassert(NtfTest::saNtfFinalize(ntfHandle), SA_AIS_OK);
  free(myNotificationParams
     .additionalText); /* allocated in fillInDefaultValues */
  test_validate(rc, SA_AIS_ERR_EXIST);
}

SaAisErrorT saNtfSubscribe(SaNtfSubscriptionIdT subscriptionId) {
  SaAisErrorT ret;
  SaNtfObjectCreateDeleteNotificationFilterT obcf;
  SaNtfNotificationTypeFilterHandlesT FilterHandles;
  memset(&FilterHandles, 0, sizeof(FilterHandles));

  safassert(saNtfObjectCreateDeleteNotificationFilterAllocate(
      ntfHandle, &obcf, 0, 0, 0, 1, 0), SA_AIS_OK);
  obcf.notificationFilterHeader.notificationClassIds->vendorId =
      SA_NTF_VENDOR_ID_SAF;
  obcf.notificationFilterHeader.notificationClassIds->majorId = 222;
  obcf.notificationFilterHeader.notificationClassIds->minorId = 222;
  FilterHandles.objectCreateDeleteFilterHandle =
      obcf.notificationFilterHandle;
  ret = NtfTest::saNtfNotificationSubscribe(&FilterHandles, subscriptionId);
  return ret;
}

void saNtfNotificationSubscribe_05(void) {
  SaNtfHandleT ntfHandle1 = 0;
  safassert(NtfTest::saNtfInitialize(&ntfHandle1, &ntfCallbacks, &ntfVersion),
      SA_AIS_OK);
  safassert(NtfTest::saNtfInitialize(&ntfHandle, &ntfCallbacks, &ntfVersion),
      SA_AIS_OK);
  safassert(saNtfSubscribe(666), SA_AIS_OK);
  safassert(NtfTest::saNtfFinalize(ntfHandle), SA_AIS_OK);

  safassert(NtfTest::saNtfInitialize(&ntfHandle, &ntfCallbacks, &ntfVersion),
      SA_AIS_OK);
  rc = saNtfSubscribe(666);
  safassert(NtfTest::saNtfFinalize(ntfHandle), SA_AIS_OK);
  safassert(NtfTest::saNtfFinalize(ntfHandle1), SA_AIS_OK);
  test_validate(rc, SA_AIS_OK);
}

__attribute__((constructor)) static void
saNtfNotificationSubscribe_constructor(void) {
  test_suite_add(10, "Consumer operations - subscribe");
  test_case_add(10, saNtfNotificationSubscribe_01,
      "saNtfNotificationSubscribe first simple SA_AIS_OK");
  test_case_add(10, saNtfNotificationSubscribe_02,
      "saNtfNotificationSubscribe null handle SA_AIS_ERR_INVALID_PARAM");
  test_case_add(10, saNtfNotificationSubscribe_03,
      "saNtfNotificationSubscribe All filter handles null  "
      "SA_AIS_ERR_INVALID_PARAM");
  test_case_add(10, saNtfNotificationSubscribe_04,
      "saNtfNotificationSubscribe subscriptionId exist SA_AIS_ERR_EXIST");
  test_case_add(10, saNtfNotificationSubscribe_05,
      "saNtfNotificationSubscribe subscriptionId if old handle has "
      "subscriptionId had been finalized SA_AIS_OK");
}
